@@ -326,7 +326,6 @@ def _generate_description_lines():
                 yield '<b>%s</b> <small>(' % name + _('no status') + ')</small>'
             else:
                 yield '<b>%s</b> <small>(' % name + _('offline') + ')</small>'
-        yield ''
 
 
 def _pick_device_with_lowest_battery():
@@ -440,6 +439,7 @@ def _update_menu_item(index, device):
     charging = device.status.get(_K.BATTERY_CHARGING)
     icon_name = _icons.battery(level, charging)
 
+    menu_item.set_label(('  ' if 0 < device.number <= 6 else '') + device.name + ': ' + device.status.to_string())
     image_widget = menu_item.get_image()
     image_widget.set_sensitive(bool(device.online))
     _update_menu_icon(image_widget, icon_name)
